Module: SideJob::Worker::OverrideMethods
- Defined in:
- lib/sidejob/worker.rb
Overview
Methods loaded last to override other included methods
Instance Method Summary collapse
-
#id ⇒ Integer
Returns the jid set by sidekiq as the job id.
Instance Method Details
#id ⇒ Integer
Returns the jid set by sidekiq as the job id
49 50 51 |
# File 'lib/sidejob/worker.rb', line 49 def id jid.to_i end |